#include <be/app/Message.h>
#include <be/app/Messenger.h>
#include <be/interface/Box.h>
#include <be/interface/RadioButton.h>
#include <be/interface/Rect.h>
#include <be/interface/StringView.h>
#include <be/interface/TextControl.h>
#include <be/storage/Entry.h>
#include <be/storage/File.h>
#include <be/support/Archivable.h>
#include <be/support/Debug.h>
#include <be/support/List.h>
#include <AudioInfo/AudioAttributes.h>
#include <EntryRefItem/EntryRefItem.h>
#include <Matcher/NameValueMatcher.h>
#include "AppDefs.h"
#include "Application.h"
#include "AppView.h"
#include "AppWindow.h"
#include "CommandConstants.h"
#include "GUIStrings.h"
#include "NAView.h"
#include "Preferences.h"

NAView::NAView(BRect frame) : AddOnView(frame,NA_VIEW_NAME)
{
	PRINT(("NAView::NAView(BRect)\n"));

	InitView();
}

NAView::~NAView()
{
	PRINT(("NAView::~NAView()\n"));
}

void
NAView::InitView()
{
	PRINT(("NAView::InitView()\n"));

	int space = 6;
	float bottom = 0;
	BRect frame(0,0,0,0);

	a2nRadioButton = new BRadioButton(frame,0,A2N_LABEL,
			new BMessage(RB_EVENT));
	a2nRadioButton->SetValue(B_CONTROL_ON);
	a2nRadioButton->ResizeToPreferred();
	a2nRadioButton->MoveTo(space,2*space);
	bottom = a2nRadioButton->Frame().bottom;

	n2aRadioButton = new BRadioButton(frame,0,N2A_LABEL,
			new BMessage(RB_EVENT));
	n2aRadioButton->ResizeToPreferred();
	n2aRadioButton->MoveTo(space,space+bottom);

	float width = 2*space +
		max_c(a2nRadioButton->Frame().Width(),n2aRadioButton->Frame().Width());

	float height = 4*space + a2nRadioButton->Frame().Height() +
		n2aRadioButton->Frame().Height();

	directionBox = new BBox(BRect(0,0,width,height),"directionBox");
	directionBox->SetLabel(DIRECTION_LABEL);
	directionBox->MoveTo(space,space);
	bottom = directionBox->Frame().bottom;

	artistStringView = new BStringView(frame,0,ARTIST_PATTERN);
	artistStringView->ResizeToPreferred();
	artistStringView->MoveTo(space,space+bottom);
	bottom = artistStringView->Frame().bottom;

	albumStringView = new BStringView(frame,0,ALBUM_PATTERN);
	albumStringView->ResizeToPreferred();
	albumStringView->MoveTo(space,space+bottom);
	bottom = albumStringView->Frame().bottom;

	titleStringView = new BStringView(frame,0,TITLE_PATTERN);
	titleStringView->ResizeToPreferred();
	titleStringView->MoveTo(space,space+bottom);
	bottom = titleStringView->Frame().bottom;

	yearStringView = new BStringView(frame,0,YEAR_PATTERN);
	yearStringView->ResizeToPreferred();
	yearStringView->MoveTo(space,space+bottom);
	bottom = yearStringView->Frame().bottom;

	commentStringView = new BStringView(frame,0,COMMENT_PATTERN);
	commentStringView->ResizeToPreferred();
	commentStringView->MoveTo(space,space+bottom);
	bottom = commentStringView->Frame().bottom;

	trackStringView = new BStringView(frame,0,TRACK_PATTERN);
	trackStringView->ResizeToPreferred();
	trackStringView->MoveTo(space,space+bottom);
	bottom = trackStringView->Frame().bottom;

	genreStringView = new BStringView(frame,0,GENRE_PATTERN);
	genreStringView->ResizeToPreferred();
	genreStringView->MoveTo(space,space+bottom);
	bottom = genreStringView->Frame().bottom;

	wildcardStringView = new BStringView(frame,0,WILDCARD_PATTERN);
	wildcardStringView->ResizeToPreferred();
	wildcardStringView->MoveTo(space,space+bottom);
	bottom = wildcardStringView->Frame().bottom;

	frame = Bounds();
	frame.InsetBy(space,0);
	frame.top = space+bottom;
	patternTextControl = new BTextControl(frame,0,PATTERN_LABEL,0,0,
			B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP);
	float divider = patternTextControl->StringWidth(PATTERN_LABEL) + 5;
	patternTextControl->SetDivider(divider);

	ResizeToPreferred();

	AddChild(directionBox);
	directionBox->AddChild(a2nRadioButton);
	directionBox->AddChild(n2aRadioButton);

	AddChild(artistStringView);
	AddChild(albumStringView);
	AddChild(titleStringView);
	AddChild(yearStringView);
	AddChild(commentStringView);
	AddChild(trackStringView);
	AddChild(genreStringView);
	AddChild(wildcardStringView);

	AddChild(patternTextControl);

	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
NAView::AttachedToWindow()
{
	AddOnView::AttachedToWindow();

	PRINT(("NAView::AttachedToWindow()\n"));

	int space = 6;

	BRect frame = Bounds();
	frame.InsetBy(space,0);
	if(frame.right > patternTextControl->Frame().right)
	{
		float diff = frame.right - patternTextControl->Frame().right;
		patternTextControl->ResizeBy(diff,0);
	}

	a2nRadioButton->SetTarget(this);
	n2aRadioButton->SetTarget(this);
}

void
NAView::GetPreferredSize(float* width, float* height)
{
	PRINT(("NAView::GetPreferredSize(float*,float*)\n"));

	*width = directionBox->Frame().right;
	*height = patternTextControl->Frame().bottom;
}

void
NAView::Hide()
{
	AddOnView::Hide();

	PRINT(("NAView::Hide()\n"));
}

void
NAView::Show()
{
	AddOnView::Show();

	PRINT(("NAView::Show()\n"));
}

void
NAView::SelectionChanged(BList* list)
{
	AddOnView::SelectionChanged(list);

	PRINT(("NAView::SelectionChanged(BList* list)\n"));

	WidgetsSetEnabled();
}

void
NAView::Apply()
{
	AddOnView::Apply();

	PRINT(("NAView::Apply()\n"));

	thread_id thread = spawn_thread(NAView::ApplyFunction,"Names & Attributes",
			B_NORMAL_PRIORITY,(void*)this);
	resume_thread(thread);
}

void
NAView::Reset()
{
	AddOnView::Reset();

	PRINT(("NAView::Reset()\n"));

	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
NAView::WidgetsSetValues()
{
	PRINT(("NAView::WidgetsSetValues()\n"));

	a2nRadioButton->SetValue(B_CONTROL_ON);
	if(prefs->Lock())
	{
		patternTextControl->SetText(prefs->GetPrefNAPattern());
		prefs->Unlock();
	}
	WidgetsRBValues();
}

void
NAView::WidgetsSetEnabled()
{
	PRINT(("NAView::WidgetsSetEnabled()\n"));

	if(selectedItems->CountItems() > 0)
	{
		a2nRadioButton->SetEnabled(true);
		n2aRadioButton->SetEnabled(true);
		patternTextControl->SetEnabled(true);
	}
	else
	{
		a2nRadioButton->SetEnabled(false);
		n2aRadioButton->SetEnabled(false);
		patternTextControl->SetEnabled(false);
	}
}

void
NAView::WidgetsRBValues()
{
	PRINT(("NAView::WidgetsRBValues()\n"));

	if(a2nRadioButton->Value() == B_CONTROL_ON)
	{
		if(!wildcardStringView->IsHidden())
		{
			wildcardStringView->Hide();
		}
	}
	else
	{
		if(wildcardStringView->IsHidden())
		{
			wildcardStringView->Show();
		}
	}
}

int32
NAView::ApplyFunction(void* args)
{
	PRINT(("NAView::ApplyFunction(void*)\n"));

	NAView* view = (NAView*)args;

	char pattern[256];
	strcpy(pattern,view->patternTextControl->Text());

	bool a2n = false;
	if(view->a2nRadioButton->Value() == B_CONTROL_ON)
	{
		a2n = true;
	}

	BView* appView = view->Window()->FindView("AppView");
	BMessenger messenger(appView);
	BMessage startMsg(START_APPLY);
	BMessage endMsg(END_APPLY);
	BMessage maxMsg(STATUS_BAR_SET_MAX_VALUE);
	BMessage updateMsg(STATUS_BAR_UPDATE);
	BMessage resetMsg(STATUS_BAR_RESET);

	messenger.SendMessage(&startMsg);

	int numSelected = view->selectedItems->CountItems();
	maxMsg.AddInt32("maxValue",numSelected);
	messenger.SendMessage(&maxMsg);
	for(int i=0;i<numSelected;i++)
	{
		EntryRefItem* refItem = (EntryRefItem*)view->selectedItems->ItemAt(i);
		entry_ref* ref = refItem->EntryRef();
		BFile id3File(ref,B_READ_WRITE);
		AudioAttributes attributes(&id3File);

		if(a2n)
		{
			const char* artist = attributes.Artist();
			if(!artist)
			{
				artist = "";
			}
			const char* album = attributes.Album();
			if(!album)
			{
				album = "";
			}
			const char* title = attributes.Title();
			if(!title)
			{
				title = "";
			}
			const char* year = attributes.Year();
			if(!year)
			{
				year = "";
			}
			const char* comment = attributes.Comment();
			if(!comment)
			{
				comment = "";
			}
			const char* track = attributes.Track();
			if(!track)
			{
				track = "";
			}
			const char* genre = attributes.Genre();
			if(!genre)
			{
				genre = "";
			}

			BString name(pattern);
			name.ReplaceAll(ARTIST_PAT,artist);
			name.ReplaceAll(ALBUM_PAT,album);
			name.ReplaceAll(TITLE_PAT,title);
			name.ReplaceAll(YEAR_PAT,year);
			name.ReplaceAll(COMMENT_PAT,comment);
			name.ReplaceAll(TRACK_PAT,track);
			name.ReplaceAll(GENRE_PAT,genre);
			BEntry entry(ref);
			entry.Rename(name.String());
			entry.GetRef(ref);
			refItem->UpdateText();
		}
		else
		{
			char name[2];
			char value[B_PATH_NAME_LENGTH+1];
			memset(name,0,2);
			memset(value,0,B_PATH_NAME_LENGTH+1);
			NameValueMatcher matcher(pattern,ref->name,'/');
			while(matcher.NextMatch(name,value) == 0)
			{
				switch(name[0])
				{
					case 'a':
						attributes.SetArtist(value);
						attributes.WriteArtist();
						break;
					case 'n':
						attributes.SetAlbum(value);
						attributes.WriteAlbum();
						break;
					case 't':
						attributes.SetTitle(value);
						attributes.WriteTitle();
						break;
					case 'y':
						attributes.SetYear(value);
						attributes.WriteYear();
						break;
					case 'c':
						attributes.SetComment(value);
						attributes.WriteComment();
						break;
					case 'k':
						attributes.SetTrack(value);
						attributes.WriteTrack();
						break;
					case 'g':
						attributes.SetGenre(value);
						attributes.WriteGenre();
						break;
					default:
						break;
				}
			}
		}
		messenger.SendMessage(&updateMsg);
	}
	if(a2n)
	{
		BMessage invalidateListMsg(INVALIDATE_LIST);
		messenger.SendMessage(&invalidateListMsg);
	}
	messenger.SendMessage(&resetMsg);
	messenger.SendMessage(&endMsg);
}

void
NAView::MessageReceived(BMessage* message)
{
	//PRINT(("NAView::MessageReceived(BMessage*)\n"));

	switch(message->what)
	{
		case RB_EVENT:
			WidgetsRBValues();
			break;
		default:
			AddOnView::MessageReceived(message);
	}
}
