#include <be/app/Message.h>
#include <be/app/Messenger.h>
#include <be/interface/Box.h>
#include <be/interface/CheckBox.h>
#include <be/interface/RadioButton.h>
#include <be/interface/Rect.h>
#include <be/interface/Window.h>
#include <be/storage/Entry.h>
#include <be/storage/File.h>
#include <be/storage/NodeInfo.h>
#include <be/storage/Path.h>
#include <be/support/Debug.h>
#include <be/support/List.h>
#include <be/support/SupportDefs.h>
#include <EntryRefItem/EntryRefItem.h>
#include <AudioInfo/AudioAttributes.h>
#include <AudioInfo/ID3Tags.h>
#include "AppDefs.h"
#include "CommandConstants.h"
#include "GUIStrings.h"
#include "TAView.h"

TAView::TAView(BRect frame) : AddOnView(frame,TA_VIEW_NAME)
{
	PRINT(("TAView::TAView(BRect)\n"));

	InitView();
}

TAView::~TAView()
{
	PRINT(("TAView::~TAView()\n"));
}

void
TAView::InitView()
{
	PRINT(("TAView::InitView()\n"));

	int space = 6;
	float bottom = 0;
	BRect frame(0,0,0,0);

	a2tRadioButton = new BRadioButton(frame,"a2tRadioButton",A2T_LABEL,NULL);
	a2tRadioButton->SetValue(B_CONTROL_ON);
	a2tRadioButton->ResizeToPreferred();
	a2tRadioButton->MoveTo(space,2*space);
	bottom = a2tRadioButton->Frame().bottom;

	t2aRadioButton = new BRadioButton(frame,"t2aRadioButton",T2A_LABEL,NULL);
	t2aRadioButton->ResizeToPreferred();
	t2aRadioButton->MoveTo(space,space+bottom);

	float width = 2*space +
		max_c(a2tRadioButton->Frame().Width(),a2tRadioButton->Frame().Width());

	float height = 4*space + 2*a2tRadioButton->Frame().Height();

	directionBox = new BBox(BRect(0,0,width,height),"directionBox");
	directionBox->SetLabel(DIRECTION_LABEL);
	directionBox->MoveTo(space,space);
	bottom = directionBox->Frame().bottom;

	artistCheckBox = new BCheckBox(frame,"artistCheckBox",ARTIST_LABEL,NULL);
	artistCheckBox->ResizeToPreferred();
	artistCheckBox->MoveTo(space,space+bottom);
	bottom = artistCheckBox->Frame().bottom;
	
	albumCheckBox = new BCheckBox(frame,"albumCheckBox",ALBUM_LABEL,NULL);
	albumCheckBox->ResizeToPreferred();
	albumCheckBox->MoveTo(space,space+bottom);
	bottom = albumCheckBox->Frame().bottom;
	
	titleCheckBox = new BCheckBox(frame,"titleCheckBox",TITLE_LABEL,NULL);
	titleCheckBox->ResizeToPreferred();
	titleCheckBox->MoveTo(space,space+bottom);
	bottom = titleCheckBox->Frame().bottom;
	
	yearCheckBox = new BCheckBox(frame,"yearCheckBox",YEAR_LABEL,NULL);
	yearCheckBox->ResizeToPreferred();
	yearCheckBox->MoveTo(space,space+bottom);
	bottom = yearCheckBox->Frame().bottom;
	
	commentCheckBox = new BCheckBox(frame,"commentCheckBox",COMMENT_LABEL,NULL);
	commentCheckBox->ResizeToPreferred();
	commentCheckBox->MoveTo(space,space+bottom);
	bottom = commentCheckBox->Frame().bottom;
	
	trackCheckBox = new BCheckBox(frame,"trackCheckBox",TRACK_LABEL,NULL);
	trackCheckBox->ResizeToPreferred();
	trackCheckBox->MoveTo(space,space+bottom);
	bottom = trackCheckBox->Frame().bottom;

	genreCheckBox = new BCheckBox(frame,"genreCheckBox",GENRE_LABEL,NULL);
	genreCheckBox->ResizeToPreferred();
	genreCheckBox->MoveTo(space,space+bottom);

	ResizeToPreferred();

	AddChild(directionBox);
	directionBox->AddChild(a2tRadioButton);
	directionBox->AddChild(t2aRadioButton);

	AddChild(artistCheckBox);
	AddChild(albumCheckBox);
	AddChild(titleCheckBox);
	AddChild(yearCheckBox);
	AddChild(commentCheckBox);
	AddChild(trackCheckBox);
	AddChild(genreCheckBox);

	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
TAView::AttachedToWindow()
{
	AddOnView::AttachedToWindow();

	PRINT(("TAView::AttachedToWindow()\n"));

	a2tRadioButton->SetTarget(this);
	t2aRadioButton->SetTarget(this);
}

void
TAView::GetPreferredSize(float* width, float* height)
{
	*width = directionBox->Frame().right;
	*height = genreCheckBox->Frame().bottom;
}

void
TAView::Hide()
{
	AddOnView::Hide();

	PRINT(("TAView::Hide()\n"));
}

void
TAView::Show()
{
	AddOnView::Show();

	PRINT(("TAView::Show()\n"));
}

void
TAView::SelectionChanged(BList* list)
{
	AddOnView::SelectionChanged(list);

	PRINT(("TAView::SelectionChanged(BList* list)\n"));

	WidgetsSetEnabled();
}

void
TAView::Apply()
{
	AddOnView::Apply();

	PRINT(("TAView::Apply()\n"));

	thread_id thread = spawn_thread(TAView::ApplyFunction,"Names & Attributes",
			B_NORMAL_PRIORITY,(void*)this);
	resume_thread(thread);
}

void
TAView::Reset()
{
	AddOnView::Reset();

	PRINT(("TAView::Reset()\n"));
	WidgetsSetValues();
	WidgetsSetEnabled();
}

void
TAView::WidgetsSetValues()
{
	PRINT(("TAView::WidgetsSetValues()\n"));

	a2tRadioButton->SetValue(B_CONTROL_ON);
	artistCheckBox->SetValue(B_CONTROL_ON);
	albumCheckBox->SetValue(B_CONTROL_ON);
	titleCheckBox->SetValue(B_CONTROL_ON);
	yearCheckBox->SetValue(B_CONTROL_ON);
	commentCheckBox->SetValue(B_CONTROL_ON);
	trackCheckBox->SetValue(B_CONTROL_ON);
	genreCheckBox->SetValue(B_CONTROL_ON);
}

void
TAView::WidgetsSetEnabled()
{
	PRINT(("TAView::WidgetsSetEnabled()\n"));

	int numSelected = selectedItems->CountItems();
	if(numSelected > 0)
	{
		a2tRadioButton->SetEnabled(true);
		t2aRadioButton->SetEnabled(true);
		artistCheckBox->SetEnabled(true);
		albumCheckBox->SetEnabled(true);
		titleCheckBox->SetEnabled(true);
		yearCheckBox->SetEnabled(true);
		commentCheckBox->SetEnabled(true);
		trackCheckBox->SetEnabled(true);
		genreCheckBox->SetEnabled(true);
	}
	else
	{
		a2tRadioButton->SetEnabled(false);
		t2aRadioButton->SetEnabled(false);
		artistCheckBox->SetEnabled(false);
		albumCheckBox->SetEnabled(false);
		titleCheckBox->SetEnabled(false);
		yearCheckBox->SetEnabled(false);
		commentCheckBox->SetEnabled(false);
		trackCheckBox->SetEnabled(false);
		genreCheckBox->SetEnabled(false);
	}
}

int32
TAView::ApplyFunction(void* args)
{
	PRINT(("TAView::ApplyFunction(void*)\n"));

	TAView* view = (TAView*)args;

	BView* appView = view->Window()->FindView("AppView");
	BMessenger messenger(appView);
	BMessage startMsg(START_APPLY);
	BMessage endMsg(END_APPLY);
	BMessage maxMsg(STATUS_BAR_SET_MAX_VALUE);
	BMessage updateMsg(STATUS_BAR_UPDATE);
	BMessage resetMsg(STATUS_BAR_RESET);

	messenger.SendMessage(&startMsg);

	bool a2t = false;
	bool doArtist = false;
	bool doAlbum = false;
	bool doTitle = false;
	bool doYear = false;
	bool doComment = false;
	bool doTrack = false;
	bool doGenre = false;

	if(view->a2tRadioButton->Value() == B_CONTROL_ON)
	{
		a2t = true;
	}

	if(view->artistCheckBox->Value() == B_CONTROL_ON)
	{
		doArtist = true;
	}

	if(view->albumCheckBox->Value() == B_CONTROL_ON)
	{
		doAlbum = true;
	}

	if(view->titleCheckBox->Value() == B_CONTROL_ON)
	{
		doTitle = true;
	}

	if(view->yearCheckBox->Value() == B_CONTROL_ON)
	{
		doYear = true;
	}

	if(view->commentCheckBox->Value() == B_CONTROL_ON)
	{
		doComment = true;
	}

	if(view->trackCheckBox->Value() == B_CONTROL_ON)
	{
		doTrack = true;
	}

	if(view->genreCheckBox->Value() == B_CONTROL_ON)
	{
		doGenre = true;
	}

	int numSelected = view->selectedItems->CountItems();
	maxMsg.AddInt32("maxValue",numSelected);
	messenger.SendMessage(&maxMsg);
	for(int i=0;i<numSelected;i++)
	{
		EntryRefItem* refItem = (EntryRefItem*)view->selectedItems->ItemAt(i);
		entry_ref* ref = refItem->EntryRef();
		BFile audioFile(ref,B_READ_WRITE);
		BNodeInfo info(&audioFile);
		char type[B_MIME_TYPE_LENGTH];
		info.GetType(type);
		if(strcmp(type,MP3_MIME_TYPE) != 0)
		{
			continue;
		}
		AudioAttributes attributes(&audioFile);
		BPath refPath(ref);
		ID3Tags tags(refPath.Path());

		if(a2t)
		{
			if(doArtist)
			{
				tags.SetArtist(attributes.Artist());
				tags.WriteArtist();
			}
			if(doAlbum)
			{
				tags.SetAlbum(attributes.Album());
				tags.WriteAlbum();
			}
			if(doTitle)
			{
				tags.SetTitle(attributes.Title());
				tags.WriteTitle();
			}
			if(doYear)
			{
				tags.SetYear(attributes.Year());
				tags.WriteYear();
			}
			if(doComment)
			{
				tags.SetComment(attributes.Comment());
				tags.WriteComment();
			}
			if(doTrack)
			{
				tags.SetTrack(attributes.Track());
				tags.WriteTrack();
			}
			if(doGenre)
			{
				tags.SetGenre(attributes.Genre());
				tags.WriteGenre();
			}
		}
		else
		{
			if(doArtist)
			{
				attributes.SetArtist(tags.Artist());
				attributes.WriteArtist();
			}
			if(doAlbum)
			{
				attributes.SetAlbum(tags.Album());
				attributes.WriteAlbum();
			}
			if(doTitle)
			{
				attributes.SetTitle(tags.Title());
				attributes.WriteTitle();
			}
			if(doYear)
			{
				attributes.SetYear(tags.Year());
				attributes.WriteYear();
			}
			if(doComment)
			{
				attributes.SetComment(tags.Comment());
				attributes.WriteComment();
			}
			if(doTrack)
			{
				attributes.SetTrack(tags.Track());
				attributes.WriteTrack();
			}
			if(doGenre)
			{
				attributes.SetGenre(tags.Genre());
				attributes.WriteGenre();
			}
		}
		messenger.SendMessage(&updateMsg);
	}
	messenger.SendMessage(&resetMsg);
	messenger.SendMessage(&endMsg);
}
