#include <string.h>
#include <be/support/Debug.h>
#include <String/String.h>
#include "ID3Tag.h"

ID3Tag::ID3Tag(FILE* file, int size, int offset)
{
	PRINT(("ID3Tag::ID3Tag(FILE*,int,int)\n"));

	this->file = file;
	this->size = size;
	this->offset = offset;

	this->value = new char[this->size+1];
	memset(this->value,0,this->size+1);
}

ID3Tag::~ID3Tag()
{
	PRINT(("ID3Tag::~ID3Tag()\n"));

	delete [] value;
	value = 0;
}

const char*
ID3Tag::Value()
{
	PRINT(("ID3Tag::Value()\n"));

	if(strcmp(this->value,"") == 0)
	{
		return 0;
	}
	return this->value;
}

void
ID3Tag::SetValue(const char* value)
{
	PRINT(("ID3Tag::SetValue(const char*)\n"));

	memset(this->value,0,size+1);

	if(value)
	{
		if(size > 1)
		{
			String tmp;
			tmp = value;
			tmp.Trim();
			strncpy(this->value,tmp.Value(),size);
		}
		else
		{
			strncpy(this->value,value,size);
		}
	}
}

int
ID3Tag::Read()
{
	PRINT(("ID3Tag::Read()\n"));

	char buf[size+1];
	memset(buf,0,size+1);

	if(fseek(file,offset,SEEK_END) < 0)
	{
		PRINT(("Error seeking tag.\n"));
		return B_ERROR;
	}

	if(fread(buf,1,size,file) != size)
	{
		PRINT(("Error reading tag.\n"));
		return B_ERROR;
	}

	SetValue(buf);

	return B_OK;
}

int
ID3Tag::Write()
{
	PRINT(("ID3Tag::Write()\n"));

	if(fseek(file,offset,SEEK_END) < 0)
	{
		PRINT(("Error seeking tag.\n"));
		return B_ERROR;
	}

	if(fwrite(value,1,size,file) != size)
	{
		PRINT(("Error writing tag.\n"));
		return B_ERROR;
	}

	return B_OK;
}

