#include <stdlib.h>
#include <string.h>
#include <be/support/Debug.h>
#include <String/String.h>
#include "GenreList.h"
#include "ID3Tags.h"

#define ID3_SIZE     128

#define TAG_SIZE     3
#define TITLE_SIZE   30
#define ARTIST_SIZE  30
#define ALBUM_SIZE   30
#define YEAR_SIZE    4
#define COMMENT_SIZE 28
#define BLANK_SIZE   1
#define TRACK_SIZE   1
#define GENRE_SIZE   1

#define TAG_OFFSET     -128
#define TITLE_OFFSET   -125
#define ARTIST_OFFSET  -95
#define ALBUM_OFFSET   -65
#define YEAR_OFFSET    -35
#define COMMENT_OFFSET -31
#define BLANK_OFFSET   -3
#define TRACK_OFFSET   -2
#define GENRE_OFFSET   -1

ID3Tags::ID3Tags(const char* filename) : AudioInfo()
{
	PRINT(("ID3Tags::ID3Tags(const char*)\n"));

	this->file = fopen(filename,"r+");
	tag = new ID3Tag(file,TAG_SIZE,TAG_OFFSET);
	artist = new ID3Tag(file,ARTIST_SIZE,ARTIST_OFFSET);
	album = new ID3Tag(file,ALBUM_SIZE,ALBUM_OFFSET);
	title = new ID3Tag(file,TITLE_SIZE,TITLE_OFFSET);
	year = new ID3Tag(file,YEAR_SIZE,YEAR_OFFSET);
	comment = new ID3Tag(file,COMMENT_SIZE,COMMENT_OFFSET);
	blank = new ID3Tag(file,BLANK_SIZE,BLANK_OFFSET);
	track = new ID3Tag(file,TRACK_SIZE,TRACK_OFFSET);
	genre = new ID3Tag(file,GENRE_SIZE,GENRE_OFFSET);

	trackValue = new char[4];

	if(HasID3Tags())
	{
		Read();
	}
}

ID3Tags::~ID3Tags()
{
	PRINT(("ID3Tags::~ID3Tags()\n"));

	fclose(file);
	delete tag;
	delete artist;
	delete album;
	delete title;
	delete year;
	delete comment;
	delete blank;
	delete track;
	delete genre;
	delete [] trackValue;
}

bool
ID3Tags::HasID3Tags()
{
	PRINT(("ID3Tags::HasID3Tags()\n"));

	bool result = false;
	if((tag->Read() != B_ERROR) && (tag->Value()))
	{
		if(strcmp(tag->Value(),"TAG") == 0)
		{
			result = true;
		}
	}

	return result;
}

int
ID3Tags::CreateID3Tags()
{
	PRINT(("ID3Tags::CreateID3Tags()\n"));

	char value[ID3_SIZE];
	memset(value,0,ID3_SIZE);
	strcpy(value,"TAG");
	ID3Tag tagarea(file,ID3_SIZE,0);
	tagarea.SetValue(value);
	return tagarea.Write();
}

const char*
ID3Tags::Artist()
{
	PRINT(("ID3Tags::Artist()\n"));

	return artist->Value();
}

const char*
ID3Tags::Album()
{
	PRINT(("ID3Tags::Album()\n"));

	return album->Value();
}

const char*
ID3Tags::Title()
{
	PRINT(("ID3Tags::Title()\n"));

	return title->Value();
}

const char*
ID3Tags::Year()
{
	PRINT(("ID3Tags::Year()\n"));

	return year->Value();
}

const char*
ID3Tags::Comment()
{
	PRINT(("ID3Tags::Comment()\n"));

	return comment->Value();
}

const char*
ID3Tags::Track()
{
	PRINT(("ID3Tags::Track()\n"));

	const char* trk = track->Value();

	if(trk)
	{
		sprintf(trackValue,"%02d",trk[0]);
		return trackValue;
	}
	return 0;
}

const char*
ID3Tags::Genre()
{
	PRINT(("ID3Tags::Genre()\n"));

	const char* value = genre->Value();
	if(value)
	{
		return GenreList::Genre((unsigned char)value[0]);
	}
	return GenreList::Genre(0);
}

void
ID3Tags::SetArtist(const char* value)
{
	PRINT(("ID3Tags::SetArtist(const char*)\n"));

	artist->SetValue(value);
}

void
ID3Tags::SetAlbum(const char* value)
{
	PRINT(("ID3Tags::SetAlbum(const char*)\n"));

	album->SetValue(value);
}

void
ID3Tags::SetTitle(const char* value)
{
	PRINT(("ID3Tags::SetTitle(const char*)\n"));

	title->SetValue(value);
}

void
ID3Tags::SetYear(const char* value)
{
	PRINT(("ID3Tags::SetYear(const char*)\n"));

	year->SetValue(value);
}

void
ID3Tags::SetComment(const char* value)
{
	PRINT(("ID3Tags::SetComment(const char*)\n"));

	comment->SetValue(value);
}

void
ID3Tags::SetTrack(const char* value)
{
	PRINT(("ID3Tags::SetTrack(const char*)\n"));

	int trk = 0;
	if(value)
	{
		trk = atoi(value);
	}
	if(trk >= 0 && trk <= 255)
	{
		char val = trk;
		track->SetValue(&val);
	}
	else
	{
		track->SetValue(0);
	}
}

void
ID3Tags::SetGenre(const char* value)
{
	PRINT(("ID3Tags::SetGenre(const char*)\n"));

	char val = GenreList::Genre(value);
	genre->SetValue(&val);
}

int
ID3Tags::ReadArtist()
{
	PRINT(("ID3Tags::ReadArtist()\n"));

	return artist->Read();
}

int
ID3Tags::ReadAlbum()
{
	PRINT(("ID3Tags::ReadAlbum()\n"));

	return album->Read();
}

int
ID3Tags::ReadTitle()
{
	PRINT(("ID3Tags::ReadTitle()\n"));

	return title->Read();
}

int
ID3Tags::ReadYear()
{
	PRINT(("ID3Tags::ReadYear()\n"));

	return year->Read();
}

int
ID3Tags::ReadComment()
{
	PRINT(("ID3Tags::ReadComment()\n"));

	return comment->Read();
}

int
ID3Tags::ReadTrack()
{
	PRINT(("ID3Tags::ReadTrack()\n"));

	if(blank->Read() != B_OK)
	{
		return B_ERROR;
	}

	if(blank->Value() == 0)
	{
		return track->Read();
	}

	track->SetValue(0);

	return B_OK;
}

int
ID3Tags::ReadGenre()
{
	PRINT(("ID3Tags::ReadGenre()\n"));

	return genre->Read();
}

int
ID3Tags::Write()
{
	PRINT(("ID3Tags::Write()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return AudioInfo::Write();
}

int
ID3Tags::WriteArtist()
{
	PRINT(("ID3Tags::WriteArtist()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return artist->Write();
}

int
ID3Tags::WriteAlbum()
{
	PRINT(("ID3Tags::WriteAlbum()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return album->Write();
}

int
ID3Tags::WriteTitle()
{
	PRINT(("ID3Tags::WriteTitle()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return title->Write();
}

int
ID3Tags::WriteYear()
{
	PRINT(("ID3Tags::WriteYear()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return year->Write();
}

int
ID3Tags::WriteComment()
{
	PRINT(("ID3Tags::WriteComment()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return comment->Write();
}

int
ID3Tags::WriteTrack()
{
	PRINT(("ID3Tags::WriteTrack()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	if(blank->Write() != B_OK)
	{
		return B_ERROR;
	}

	return track->Write();
}

int
ID3Tags::WriteGenre()
{
	PRINT(("ID3Tags::WriteGenre()\n"));

	if(!HasID3Tags())
	{
		CreateID3Tags();
	}
	
	return genre->Write();
}

